@@ -1,4 +1,4 @@
-/* GMM (C) 2018 Stephane Charette <stephanecharette@gmail.com>
+/* GMM (C) 2018-2019 Stephane Charette <stephanecharette@gmail.com>
  * $Id: SerialCommWin32.cpp 2629 2018-10-12 07:02:08Z stephane $
  */
 
@@ -15,6 +15,8 @@
 static const std::vector<std::string> sample_data =
 {
 #if 0
+	"14.1\t1", "4.1\r\n", "1", "4.2", "\t14", ".2\r", "\n", "1", "4", ".", "3", "\t", "1", "4", ".", "3", "\r", "junkxxx", "\n", "afasdf", "jjj", "afjafjweofnwfwifjw", "14.2 afasf\t\t\tasfasfasfasfasf", "asfasfowfjworj", "afafasfasfasfasfsafafafasfafasfasf", "afdasfaf", "adfasfaf", "afdasfasf", "afasf", "asfasfa", "awfaf", "awfawf"
+#elif 0
 	"14.1\t14.1\r\n", "14.2\t14.2\r\n", "14.3\t14.3\r\n"
 #else
 	"14.8\t14.2\r\n", "15.6\t15.0\r\n", "16.0\t15.1\r\n", "15.4\t14.8\r\n", "14.2\t13.5\r\n", "16.4\t15.6\r\n", "28.0\t21.2\r\n", "15.3\t14.4\r\n", "14.4\t13.6\r\n", "14.8\t13.5\r\n",
