@@ -1,5 +1,5 @@
-/* GMM (C) 2018 Stephane Charette <stephanecharette@gmail.com>
- * $Id: TabCanvas.cpp 2710 2019-01-25 15:41:56Z stephane $
+/* GMM (C) 2018-2019 Stephane Charette <stephanecharette@gmail.com>
+ * $Id: TabCanvas.cpp 2718 2019-02-07 07:08:42Z stephane $
  */
 
 #include "GMM.hpp"
@@ -154,8 +154,6 @@ TabCanvas::TabCanvas(const std::string & name) :
 	// re-populate the kiln, run, and package numbers
 	update_kiln_run_pkg();
 
-	read_all_mc_files();
-
 	// add the chart, and start the timer to redraw it every few seconds
 	addAndMakeVisible(chart);
 	timerCallback();
@@ -683,16 +681,16 @@ void TabCanvas::device_thread(void)
 	auto time_now	= std::chrono::high_resolution_clock::now();
 	auto next_read	= time_now + interval;
 
+	std::string str;
 	while (threadShouldExit() == false)
 	{
-		std::string str;
 		if (run_simulation)
 		{
-			str = get_simulated_string(read_buffer);
+			str += get_simulated_string(read_buffer);
 		}
 		else
 		{
-			str = read_from_device(read_buffer);
+			str += read_from_device(read_buffer);
 		}
 
 		if (str.empty() == false && threadShouldExit() == false)
@@ -752,9 +750,15 @@ void TabCanvas::device_thread(void)
 						write_mc_file();
 					}
 				}
+
+				// consume everything up to the end of the matched string, but keep trailing characters
+				// that were not part of the match in case we captured the start of the next value
+				str = matches.suffix().str();
 			}
-			else
+
+			if (str.length() > read_len * 10)
 			{
+				// something is wrong if we get here, the string must contain a bunch of garbage
 				std::string msg;
 				for (const char c : str)
 				{
@@ -762,7 +766,8 @@ void TabCanvas::device_thread(void)
 					if (i > 32 && i < 127)	msg += c; // normal ASCII character
 					else					msg += "<" + std::to_string(i) + ">"; // non-ASCII printable character
 				}
-				LOG_MSG(device_name << ": failed to parse string: len=" << str.size() << ": \"" << msg << "\"");
+				LOG_MSG(device_name << ": purging long/invalid string: len=" << str.size() << ": \"" << msg << "\"");
+				str.clear();
 			}
 		}
 
@@ -809,34 +814,64 @@ void TabCanvas::write_mc_file(void)
 	File dir(dirname);
 	dir.createDirectory();
 
-	const ScopedLock scope_lock(numbers_and_calculations_mutex);
+	std::string filename;
 
-	const std::string filename = dir.getChildFile(kiln_num_editor.getText() + "_" + run_editor.getText() + pkg_editor.getText() + ".mc").getFullPathName().toStdString();
-	LOG_MSG("prefix=" << prefix << ": writing data out to: " << filename << " (pkg size=" << pkg_doubles.size() << ", pkg avg=" <<  pkg_average << ")");
-
-	// LINE #1:
-
-	std::ofstream fs(filename);
-	fs << std::fixed << std::setw(4) << std::setfill('0') << pkg_doubles.size() << ",";
-
-	for (size_t idx = 0; idx < number_of_buckets; idx++)
+	if (true)
 	{
-		fs << std::setw(4) << std::setfill('0') << pkg_moisture_bucket[idx] << ",";
+		const std::string k = kiln_num_editor.getText().toStdString();
+		const std::string r = run_editor.getText().toStdString();
+		const std::string p = pkg_editor.getText().toStdString();
+
+		filename = dir.getChildFile(k + "_" + r + p + ".mc").getFullPathName().toStdString();
+		LOG_MSG("prefix=" << prefix << ": writing " << pkg_doubles.size() << " boards to " << filename << " (k=" << k << ", r=" << r << ", p=" << p << ", pkg avg=" << pkg_average << ", run avg=" << run_average << ")");
+
+		const ScopedLock scope_lock(numbers_and_calculations_mutex);
+
+		// LINE #1:
+
+		std::ofstream fs(filename);
+		fs << std::fixed << std::setw(4) << std::setfill('0') << pkg_doubles.size() << ",";
+
+		for (size_t idx = 0; idx < number_of_buckets; idx++)
+		{
+			fs << std::setw(4) << std::setfill('0') << pkg_moisture_bucket[idx] << ",";
+		}
+
+		fs << "000,000,";	// unknown fields
+
+		fs << pkg_std_dev << "," << pkg_average << ",000" << std::endl;
+
+		// LINE #2:
+
+		for (const double d : pkg_doubles)
+		{
+			fs << std::setw(4) << std::setfill('0') << std::setprecision(1) << d << ",";
+		}
+		fs << "." << std::endl;
 	}
 
-	fs << "000,000,";	// unknown fields
-
-	fs << pkg_std_dev << "," << pkg_average << ",000" << std::endl;
-
-	// LINE #2:
-
-	for (const double d : pkg_doubles)
+	#if 1
+	try
 	{
-		fs << std::setw(4) << std::setfill('0') << std::setprecision(1) << d << ",";
+		std::string text = Lox::readFile(filename);
+		// get rid of \r and keep just the \n
+		while (true)
+		{
+			size_t pos = text.find("\r");
+			if (pos == std::string::npos)
+			{
+				break;
+			}
+			text.erase(pos, 1);
+		}
+		text = "----------\n" + text + "----------";
+		LOG_MSG("prefix=" << prefix << ": " << filename << ":" << std::endl << text);
 	}
-	fs << "." << std::endl;
-
-	LOG_MSG("prefix=" << prefix << ": done writing data to " << filename);
+	catch (...)
+	{
+		// do nothing, we don't care if something failed while re-reading the file (but why did it fail?)
+	}
+	#endif
 
 	return;
 }
@@ -852,24 +887,50 @@ void TabCanvas::write_summary_file(void)
 	File dir(dirname);
 	dir.createDirectory();
 
-	const ScopedLock scope_lock(numbers_and_calculations_mutex);
+	std::string filename;
 
-	const std::string filename = dir.getChildFile(kiln_num_editor.getText() + "_" + run_editor.getText() + "000.mc").getFullPathName().toStdString();
-	LOG_MSG("prefix=" << prefix << ": writing summary data to: " << filename << " (pkg size=" << pkg_doubles.size() << ", pkg avg=" << pkg_average << ")");
-
-	std::ofstream fs(filename);
-	fs << run_doubles.size() << ",";
-
-	for (size_t idx = 0; idx < number_of_buckets; idx++)
+	if (true)
 	{
-		fs << run_moisture_bucket[idx] << ",";
+		const ScopedLock scope_lock(numbers_and_calculations_mutex);
+
+		filename = dir.getChildFile(kiln_num_editor.getText() + "_" + run_editor.getText() + "000.mc").getFullPathName().toStdString();
+		LOG_MSG("prefix=" << prefix << ": writing summary data to " << filename << " (run size=" << run_doubles.size() << ", run avg=" << run_average << ")");
+
+		std::ofstream fs(filename);
+		fs << run_doubles.size() << ",";
+
+		for (size_t idx = 0; idx < number_of_buckets; idx++)
+		{
+			fs << run_moisture_bucket[idx] << ",";
+		}
+
+		fs << "000,000,";	// unknown fields
+
+		fs << std::fixed << run_std_dev << "," << run_average << ",0.0,0.0" << std::endl;
 	}
 
-	fs << "000,000,";	// unknown fields
-
-	fs << std::fixed << run_std_dev << "," << run_average << ",0.0,0.0" << std::endl;
-
-	LOG_MSG("prefix=" << prefix << ": done writing summary data to " << filename);
+	#if 1
+	try
+	{
+		std::string text = Lox::readFile(filename);
+		// get rid of \r and keep just the \n
+		while (true)
+		{
+			size_t pos = text.find("\r");
+			if (pos == std::string::npos)
+			{
+				break;
+			}
+			text.erase(pos, 1);
+		}
+		text = "----------\n" + text + "----------";
+		LOG_MSG("prefix=" << prefix << ": " << filename << ":" << std::endl << text);
+	}
+	catch (...)
+	{
+		// do nothing, we don't care if something failed while re-reading the file (but why did it fail?)
+	}
+	#endif
 
 	return;
 }
@@ -910,8 +971,17 @@ void TabCanvas::read_all_mc_files(void)
 		const int package_index = *iter;
 		packages.erase(iter);
 
+		std::stringstream filename_ss;
+		filename_ss
+			<< current_kiln << "_"
+			<< std::setfill('0') << std::setw(3) << current_run
+			<< std::setfill('0') << std::setw(3) << package_index
+			<< ".mc";
+
 		if (need_to_clear_out_old_package_values)
 		{
+			LOG_MSG("prefix=" << prefix << ": clearing out all package buckets and " << pkg_doubles.size() << " board entries to load k=" << current_kiln << " << r=" << current_run << " p=" << package_index << " from " << filename_ss.str() << " (run so far contains " << run_doubles.size() << " boards)");
+
 			// we're starting to read a new package, so clear out the package-specific values
 			for (size_t idx = 0; idx < number_of_buckets; idx++)
 			{
@@ -923,12 +993,6 @@ void TabCanvas::read_all_mc_files(void)
 			need_to_clear_out_old_package_values = false;
 		}
 
-		std::stringstream filename_ss;
-		filename_ss
-			<< current_kiln << "_"
-			<< std::setfill('0') << std::setw(3) << current_run
-			<< std::setfill('0') << std::setw(3) << package_index
-			<< ".mc";
 		File file = dir.getChildFile(filename_ss.str());
 
 		if (file.existsAsFile() == false)
@@ -991,15 +1055,33 @@ void TabCanvas::read_all_mc_files(void)
 		if (run_doubles.empty() == false)
 		{
 			run_average = run_sum / static_cast<double>(run_doubles.size());
-			LOG_MSG("prefix=" << prefix << ": RUN: filename=" << filename << ": imported " << counter << " existing entries, run average is " << run_average);
+			LOG_MSG("prefix=" << prefix << ": RUN: filename=" << filename << ": imported " << counter << " existing entries, run average is " << run_average << ", run contains a total of " << run_doubles.size() << " entries");
 		}
 		if (pkg_doubles.empty() == false)
 		{
 			pkg_average = pkg_sum / static_cast<double>(pkg_doubles.size());
-			LOG_MSG("prefix=" << prefix << ": PKG: filename=" << filename << ": imported " << counter << " existing entries, pkg average is " << pkg_average);
+			LOG_MSG("prefix=" << prefix << ": PKG: filename=" << filename << ": imported " << counter << " existing entries, pkg average is " << pkg_average << ", pkg contains a total of " << pkg_doubles.size() << " entries");
 		}
 	}
 
+	LOG_MSG("prefix=" << prefix << ": import has finished loading " << run_doubles.size() << " boards");
+
+	#if 1
+	std::string text = std::to_string(run_doubles.size()) + " boards in run: k=" + std::to_string(current_kiln) + " r=" + std::to_string(current_run) + ": ";
+	for (const double & d : run_doubles)
+	{
+		text += Lox::Numbers::format(d, 1) + ", ";
+	}
+	LOG_MSG("prefix=" << prefix << ": " << text);
+
+	text = std::to_string(pkg_doubles.size()) + " boards in pkg: k=" + std::to_string(current_kiln) + " r=" + std::to_string(current_run) + ", p=" + std::to_string(current_pkg) + ": ";
+	for (const double & d : pkg_doubles)
+	{
+		text += Lox::Numbers::format(d, 1) + ", ";
+	}
+	LOG_MSG("prefix=" << prefix << ": " << text);
+	#endif
+
 	return;
 }
 
@@ -1138,6 +1220,8 @@ void TabCanvas::update_kiln_run_pkg(void)
 	bool run_needs_to_be_reset = false;
 	bool pkg_needs_to_be_reset = false;
 
+	/* New logic.  If anything has changed, then reload everything.
+	 *
 	if (current_kiln	> 0 &&
 		current_run		> 0 &&
 		current_pkg		> 0)
@@ -1150,11 +1234,25 @@ void TabCanvas::update_kiln_run_pkg(void)
 		}
 		if (current_run != new_run || current_kiln != new_kiln)
 		{
-			// if the run changes, then both the run and the package both need to be reset
+			// if the run changes, then both the run and the package need to be reset
 			pkg_needs_to_be_reset = true;
 			run_needs_to_be_reset = true;
 		}
 	}
+	 *
+	 */
+	need_to_output_mc_file = true;
+	run_needs_to_be_reset = true;
+	pkg_needs_to_be_reset = true;
+
+	if (current_kiln	== 0 &&
+		current_run		== 0 &&
+		current_pkg		== 0)
+	{
+		// we must be starting up -- no need to output the .mc files in this case
+		LOG_MSG("prefix=" << prefix << ": skipping output of .mc files since kiln, run, and package numbers are all zero");
+		need_to_output_mc_file = false;
+	}
 
 	LOG_MSG("prefix=" << prefix << ": need to update the kiln, run, or package number");
 	LOG_MSG("prefix=" << prefix << ": -> current k=" << current_kiln	<< " r=" << current_run	<< " p=" << current_pkg	);
@@ -1168,37 +1266,40 @@ void TabCanvas::update_kiln_run_pkg(void)
 		const ScopedLock scope_lock(numbers_and_calculations_mutex);
 		write_mc_file();
 		write_summary_file();
+	}
 
-		for (size_t idx = 0; idx < number_of_buckets; idx++)
-		{
-			if (pkg_needs_to_be_reset)
-			{
-				pkg_moisture_bucket[idx] = 0;
-			}
-			if (run_needs_to_be_reset)
-			{
-				run_moisture_bucket[idx] = 0;
-			}
-		}
-
+	for (size_t idx = 0; idx < number_of_buckets; idx++)
+	{
 		if (pkg_needs_to_be_reset)
 		{
-			pkg_doubles.clear();
-			pkg_sum		= 0.0;
-			pkg_average	= 0.0;
-			pkg_std_dev	= 0.0;
-			time_since_last_board_warning = 0;
-			number_of_boards_when_warning_was_last_issued = 0;
+			pkg_moisture_bucket[idx] = 0;
 		}
 		if (run_needs_to_be_reset)
 		{
-			run_doubles.clear();
-			run_sum		= 0.0;
-			run_average	= 0.0;
-			run_std_dev	= 0.0;
+			run_moisture_bucket[idx] = 0;
 		}
 	}
 
+	if (pkg_needs_to_be_reset)
+	{
+		LOG_MSG("prefix=" << prefix << ": reset all PKG values to zero");
+		pkg_doubles.clear();
+		pkg_sum		= 0.0;
+		pkg_average	= 0.0;
+		pkg_std_dev	= 0.0;
+		time_since_last_board_warning = 0;
+		number_of_boards_when_warning_was_last_issued = 0;
+	}
+
+	if (run_needs_to_be_reset)
+	{
+		LOG_MSG("prefix=" << prefix << ": reset all RUN values to zero");
+		run_doubles.clear();
+		run_sum		= 0.0;
+		run_average	= 0.0;
+		run_std_dev	= 0.0;
+	}
+
 	std::string kiln_str = std::to_string(new_kiln);
 	std::string run_str = std::to_string(new_run);
 	std::string pkg_str = std::to_string(new_pkg);
@@ -1241,6 +1342,7 @@ void TabCanvas::update_kiln_run_pkg(void)
 	// if the run was reset, re-read all the new files (if any) now that we've updated the run and package numbers
 	if (run_needs_to_be_reset)
 	{
+		LOG_MSG("prefix=" << prefix << ": significant change detected, so re-read the relevant .mc files");
 		read_all_mc_files();
 	}
 
